<?php
class PieChart extends Chart {
  
  var $_max_degrees = 360;
  var $_slice_degree_offset = -90;
  
  function PieChart($width, $height, $sections=Array()) {
    $this->_width = (int)$width;
    $this->_height = (int)$height;
    $this->_sections = (array)$sections;
    
    $this->setBackgroundColor('transparent');
    $this->setBorderColor('FFFFFF');
    $this->setBorderWidth(2);
    $this->setFontSize(3);
  }
  
  function generate() {
    // fake antialiasing
    $this->_width *= 2;
    $this->_height *= 2;
    $this->_border_width *= 2;
    
    // initialize some vars
    $image_center_x = $this->_width / 2;
    $image_center_y = $this->_height / 2;
    $uses_alpha = false;
    if (empty($this->_background_color) || strtolower($this->_background_color) == 'transparent') {
      $uses_alpha = true;
    }
    
    // initialize image
    $piechart_image = imagecreatetruecolor($this->_width, $this->_height);
    
    // draw background
    if ($uses_alpha) {
      imagealphablending($piechart_image, false);
      $background_color = imagecolorallocatealpha($piechart_image, 0, 0, 0, 127);
      imagefill($piechart_image, 0, 0, $background_color);
      imagealphablending($piechart_image, true);
    } else {
      $background_color_array = $this->convertRGB($this->_background_color);
      $background_color = imagecolorallocate($piechart_image, $background_color_array['red'], $background_color_array['green'], $background_color_array['blue']);
      imagefill($piechart_image, 0, 0, $background_color);
    }
    
    // draw border
    if ($this->_border_width > 0) {
      $border_color_array = $this->convertRGB($this->_border_color);
      $border_color = imagecolorallocate($piechart_image, $border_color_array['red'], $border_color_array['green'], $border_color_array['blue']);
      imageellipse($piechart_image, $image_center_x, $image_center_y, $this->_width, $this->_height, $border_color);
      imagefill($piechart_image, $image_center_x, $image_center_y, $border_color);
    }
    
    // draw slices
    $slice_start = 0;
    $color_index = 0;
    $slice_colors = Array();
    foreach ($this->_sections as $slice) {
      $slice_percent = $slice[0];
      $slice_color = $slice[1];
      
      $slice_color_array = $this->convertRGB($slice_color);
      $slice_colors[$color_index] = imagecolorallocate($piechart_image, $slice_color_array['red'], $slice_color_array['green'], $slice_color_array['blue']);
      
      $slice_end = $slice_start + ($slice_percent / 100) * $this->_max_degrees;
      
      imagefilledarc($piechart_image, $image_center_x, $image_center_y, $this->_width - $this->_border_width*2, $this->_height - $this->_border_width*2, $slice_start + $this->_slice_degree_offset, $slice_end + $this->_slice_degree_offset, $slice_colors[$color_index], IMG_ARC_PIE);
      
      $slice_start = $slice_end;
      $color_index++;
    }
    
    // draw slice borders
    if ($this->_border_width > 0) {
      imagesetthickness($piechart_image, $this->_border_width);
      $degrees_offset = $this->_slice_degree_offset;
      foreach ($this->_sections as $slice) {
        $slice_percent = $slice[0];
        
        $end_degrees = ($slice_percent / 100) * $this->_max_degrees;
        $end_degrees += $degrees_offset;
        
        $theta = deg2rad($end_degrees);
        
        $start_x = $image_center_x;
        $start_y = $image_center_y;
        $radius_x = $this->_width / 2;
        $radius_y = $this->_height / 2;
        $end_x = $start_x + ($radius_x * cos($theta));
        $end_y = $start_y + ($radius_y * sin($theta));
        
        imageline($piechart_image, $start_x, $start_y, $end_x, $end_y, $border_color);
        
        $degrees_offset += ($end_degrees - $degrees_offset);
      }
    }
    
    if ($uses_alpha) {
      imagealphablending($piechart_image, false);
      imagesavealpha($piechart_image, true);
    }
    
    // fake antialiasing
    $this->_width /= 2;
    $this->_height /= 2;
    $image_center_x /= 2;
    $image_center_y /= 2;
    $piechart_image_antialiased = imagecreatetruecolor($this->_width, $this->_height);
    if ($uses_alpha) imagealphablending($piechart_image_antialiased, false);
    
    imagecopyresampled($piechart_image_antialiased, $piechart_image, 0, 0, 0, 0, $this->_width, $this->_height, $this->_width*2, $this->_height*2);
    
    // draw text
    $color_index = 0;
    $text_colors = Array();
    $degrees_offset = $this->_slice_degree_offset;
    foreach ($this->_sections as $slice) {
      $slice_percent = $slice[0];
      $text = $slice[2];
      $text_color = $slice[3];
      
      $text_color_array = $this->convertRGB($text_color);
      $text_colors[$color_index] = imagecolorallocate($piechart_image_antialiased, $text_color_array['red'], $text_color_array['green'], $text_color_array['blue']);
      
      // need to calculate line in the middle of the slice to determine text position
      $end_degrees = ($slice_percent / 100) * $this->_max_degrees;
      $end_degrees += $degrees_offset;
      
      $end_degrees_middle = $end_degrees - (($slice_percent / 100) * $this->_max_degrees) / 2;
      
      $theta = deg2rad($end_degrees_middle);
      
      $start_x = $image_center_x;
      $start_y = $image_center_y;
      $radius_x = $this->_width / 4; // dividing by 4 because we want half the radius to find the center of the slice
      $radius_y = $this->_height / 4;
      $end_x = $start_x + ($radius_x * cos($theta));
      $end_y = $start_y + ($radius_y * sin($theta));
      $text_x = $end_x;
      $text_y = $end_y;
      
      $font_width = imagefontwidth($this->_font_size);
      $text_x -= (strlen($text) / 2) * $font_width;
      
      $font_height = imagefontheight($this->_font_size);
      $text_y -= $font_height/2;
      
      imagestring($piechart_image_antialiased, $this->_font_size, $text_x, $text_y, $text, $text_colors[$color_index]);
      
      $color_index++;
      $degrees_offset += ($end_degrees - $degrees_offset);
    }
    
    if ($uses_alpha) {
      imagealphablending($piechart_image_antialiased, false);
      imagesavealpha($piechart_image_antialiased, true);
    }
    
    header('Content-type: image/png');
    imagepng($piechart_image_antialiased);
    imagedestroy($piechart_image_antialiased);
    imagedestroy($piechart_image);
  }
  
}
?>
